#! /usr/bin/env python
# -*- coding: utf-8 -*-
####################
# Indigo Blink Plugin
# Copyright (c) 2022 Alan Carter. All rights reserved.
#
# Blinkpy Library copyright (c) 2017 Kevin Fronczak
#


################################################################################
# Imports
################################################################################
from blinkpy.blinkpy import Blink
from blinkpy.auth import Auth
import datetime
from time import strftime


################################################################################
# Globals
################################################################################

########################################
def updateVar(name, value, folder=0):
    if name not in indigo.variables:
        indigo.variable.create(name, value=value, folder=folder)
    else:
        indigo.variable.updateValue(name, value)

################################################################################
class Plugin(indigo.PluginBase):
    ########################################
    # Class properties
    ########################################
    
    ########################################
    def __init__(self, pluginId, pluginDisplayName, pluginVersion, pluginPrefs): 
        indigo.PluginBase.__init__(self, pluginId, pluginDisplayName, pluginVersion, pluginPrefs)
        self.blinkName = pluginPrefs.get("blinkName", "")
        self.blinkPass = pluginPrefs.get("blinkPass", "")
        self.blinkKey = pluginPrefs.get("blinkKey", "")
        self.snapPath = pluginPrefs.get("snapshotPath")
        self.omitTimestamp = pluginPrefs.get("omitTimestamp", False)
        self.logComms = pluginPrefs.get("logComms", False)
        self.connected = False
        self.loopTimer = 0
        self.actionTimer = 0

    
    ########################################
    def __del__(self):
        indigo.PluginBase.__del__(self)
        
    ########################################
    def startup(self):
        indigo.server.log("Starting Blink plugin")
        if self.blinkName == "":
            indigo.server.log("Please enter Blink account user name in Config and restart plugin", isError=True)
            return
        if self.blinkPass == "":
            indigo.server.log("Please enter Blink account password in Config and restart plugin", isError=True)
            return
        if self.logComms:
            indigo.server.log("Blink comms logging is on")
        else:
            indigo.server.log("Blink comms logging is off")
        self.firstTime = True         
        self.connect()
        
        
    def deviceStartComm(self, dev):
        dev.stateListOrDisplayStateIdChanged()
        return
        

    def shutdown(self):
        indigo.server.log("Stopping Blink plugin")

        
    ########################################
    def runConcurrentThread(self):
        indigo.server.log("Starting Blink monitoring thread")
        while True:
            if self.connected:
                if self.firstTime:
                	self.createDevices()
                	self.firstTime = False
                self.loopTimer += 1
                if self.actionTimer > 0:
                	self.actionTimer -= 1
                if self.loopTimer == 9:
                	self.loopTimer = 0
                	self.updateCams()
                self.sleep(1)
                
            else:
                self.sleep(600)

        
    
    ########################################
    # Called when user prefs are changed
    
    def closedPrefsConfigUi(self, valuesDict, userCancelled):
        # If the user saved preferences, update logging parameters
        if userCancelled == False:
            self.blinkKey = valuesDict.get("blinkKey", "")
            omitTimestamp = self.omitTimestamp
            self.omitTimestamp = valuesDict.get("omitTimestamp", False)
            if self.omitTimestamp != omitTimestamp:
                if self.omitTimestamp:
                    indigo.server.log("Timestamp will be omitted from snapshot file names")
                else:
                    indigo.server.log("Timestamp will be added to snapshot file names")            
            logComms = self.logComms
            self.logComms = valuesDict.get("logComms", False)
            if self.logComms != logComms:
                if self.logComms:
                    indigo.server.log("Blink comms logging is on")
                else:
                    indigo.server.log("Blink comms logging is off")
                

    ########################################
    # Blink specific functions

    def connect(self):
        indigo.server.log("Connecting to Blink servers")
        self.blink = Blink()
        auth = Auth({"username": self.blinkName, "password": self.blinkPass}, no_prompt=True)
        self.blink.auth = auth
        self.blink.start()
        self.sleep(5)
        if self.blinkKey == "":
            indigo.server.log("Enter Blink key into Config and restart plugin")
            return
        auth.send_auth_key(self.blink, self.blinkKey)
        self.blink.setup_post_verify()
        self.connected = True


    def createDevices(self):
        self.blink.refresh()
        for name, module in self.blink.sync.items():
            modAddress = str(module.attributes["id"])
            device = None
            for dev in indigo.devices.iter("self"):
                if dev.address == modAddress:
                    device = indigo.devices[dev.name]
            if device == None:
                indigo.server.log("Creating Blink Sync device for %s" % (name))
                nameC = "Blink Sync (%s)" % (name)
                device = indigo.device.create(protocol=indigo.kProtocol.Plugin, address=modAddress, name=nameC, pluginId="com.racarter.indigoplugin.blink", deviceTypeId="blinksync", props={})
        for name, camera in self.blink.cameras.items():
            camAddress = camera.attributes["camera_id"]
            device = None
            for dev in indigo.devices.iter("self"):
                if dev.address == camAddress:
                    device = indigo.devices[dev.name]
            if device == None:
                indigo.server.log("Creating Blink camera device for %s" % (name))
                nameC = "Blink Camera (%s)" % (name)
                device = indigo.device.create(protocol=indigo.kProtocol.Plugin, address=camAddress, name=nameC, pluginId="com.racarter.indigoplugin.blink", deviceTypeId="blinkcam", props={})


    def updateCams(self):
        self.blink.refresh()
        for name, camera in self.blink.cameras.items():
            camName = name
            if self.logComms:
                indigo.server.log("Camera Data:")
                indigo.server.log(str(camera.attributes))
            if camera.attributes["motion_enabled"]:
                md_state = "on"
            else:
                md_state = "off"
            device = None
            for dev in indigo.devices.iter("self"):
                if dev.address == camera.attributes["camera_id"]:
                    device = indigo.devices[dev.name]
            if device != None:
                keyValueList = [
                    {'key':'name', 'value':camName},
                    {'key':'camera_id', 'value':camera.attributes["camera_id"]},
                    {'key':'serial', 'value':camera.attributes["serial"]},
                    {'key':'temperature_f', 'value':camera.attributes["temperature"]},
                    {'key':'temperature', 'value':camera.attributes["temperature_c"]},
                    {'key':'battery', 'value':camera.attributes["battery"]},
                    {'key':'battery_voltage', 'value':camera.attributes["battery_voltage"]},
                    {'key':'thumbnail', 'value':camera.attributes["thumbnail"]},
                    {'key':'video', 'value':camera.attributes["video"]},
                    {'key':'motion_enabled', 'value':camera.attributes["motion_enabled"]},
                    {'key':'md_state', 'value':md_state},
                    {'key':'motion_detected', 'value':camera.attributes["motion_detected"]},
                    {'key':'wifi_strength', 'value':camera.attributes["wifi_strength"]},
                    {'key':'network_id', 'value':camera.attributes["network_id"]},
                    {'key':'sync_module', 'value':camera.attributes["sync_module"]},
                    {'key':'last_record', 'value':camera.attributes["last_record"]},
                    {'key':'type', 'value':camera.attributes["type"]},
                    {'key':'last_update_time', 'value':strftime("%d-%m-%y %H:%M:%S")},
                ]
                device.updateStatesOnServer(keyValueList)
                if camera.attributes["motion_detected"]:
                    device.updateStateImageOnServer(indigo.kStateImageSel.MotionSensorTripped)
                else:
                    device.updateStateImageOnServer(indigo.kStateImageSel.MotionSensor)
        
        for name, module in self.blink.sync.items():
            if self.logComms:
                indigo.server.log("Sync Module Data:")
                indigo.server.log(str(module.attributes))
            modAddress = str(module.attributes["id"])
            sync = self.blink.sync[name]
            if module.attributes["status"] == "online":
            	if sync.arm:
                	armState = "Armed"
            	else:
                	armState = "Disarmed"
            else:
            	armState = "Offline"
            device = None
            for dev in indigo.devices.iter("self"):
                if dev.address == modAddress:
                    device = indigo.devices[dev.name]
            if device != None:
                keyValueList = [
                    {'key':'name', 'value':name},
                    {'key':'id', 'value':modAddress},
                    {'key':'state', 'value':armState},
                    {'key':'serial', 'value':module.attributes["serial"]},
                    {'key':'network_id', 'value':module.attributes["network_id"]},
                    {'key':'status', 'value':module.attributes["status"]},
                    {'key':'region_id', 'value':module.attributes["region_id"]},
                    {'key':'last_update_time', 'value':strftime("%d-%m-%y %H:%M:%S")},
                ]
                device.updateStatesOnServer(keyValueList)            


    def armSync(self, modName):
        self.blink.refresh()
        sync = self.blink.sync[modName]
        if sync.arm:
            indigo.server.log("%s already armed - no action taken" % modName)
        else:
            self.blink.sync[modName].arm = True
            indigo.server.log("%s armed" % modName)


    def disarmSync(self, modName):
        self.blink.refresh()
        sync = self.blink.sync[modName]
        if sync.arm:
            self.blink.sync[modName].arm = False
            indigo.server.log("%s disarmed" % modName)
        else:
            indigo.server.log("%s already disarmed - no action taken" % modName)
    
    
    def armCamera(self, camName):
        self.blink.refresh()
        camera = self.blink.cameras[camName]
        if camera.arm:
             indigo.server.log("%s camera already armed - no action taken" % camName)
        else:
            camera.arm = True
            indigo.server.log("%s camera armed" % camName)
            

    def disarmCamera(self, camName):
        self.blink.refresh()
        camera = self.blink.cameras[camName]
        if camera.arm:
            camera.arm = False
            indigo.server.log("%s camera disarmed" % camName)
        else:
             indigo.server.log("%s camera already disarmed - no action taken" % camName)
            

    def getSnapshot(self, camName):
        self.blink.refresh()
        camera = self.blink.cameras[camName]
        camera.snap_picture()
        self.blink.refresh()
        self.sleep(2)
        if self.snapPath == "":
            indigo.server.log("Please specify path to snapshot folder in config and try again", isError=True)
        else:
            if self.omitTimestamp:
                snapPath = self.snapPath+"/"+camName+".jpg"
            else:
                snapPath = self.snapPath+"/"+camName+" "+strftime("%d-%m-%y %H:%M:%S")+".jpg"
            camera.image_to_file(snapPath)


    def getVideo(self, mins):
        self.blink.refresh()
        cutoff = datetime.datetime.now() - datetime.timedelta(minutes=mins)
        cutoffStr = "'"+cutoff.strftime("%Y/%m/%d %H:%M")+"'"
        if self.snapPath == "":
            indigo.server.log("Please specify path to snapshot folder in config and try again", isError=True)
        else:
            self.blink.download_videos(self.snapPath, since=cutoffStr, delay=2)




    ########################################
    # Action Item functions
    ######################
    
    def setArmAction(self, pluginAction, actionId):
        """ Action item to arm/disarm a specified sync module """
        devToProcess = int(pluginAction.props["syncmodule"])
        device = None
        for dev in indigo.devices.iter("self"):
            if dev.id == devToProcess:
                device = indigo.devices[dev.name]
                if device != None:
                    if pluginAction.props["armType"] == "disarm":
                        self.disarmSync(device.states["name"])
                    elif pluginAction.props["armType"] == "arm":
                        self.armSync(device.states["name"])
                    self.sleep(5)
                    return True
    

    def setArmCam(self, pluginAction, actionId):
        """ Menu item to arm/disarm a specified camera """
        while self.actionTimer != 0:
            self.sleep(1)
        self.actionTimer = 10   # Wait 10 seconds between commands or they can be missed
        devToProcess = int(pluginAction.props["camera"])
        device = None
        for dev in indigo.devices.iter("self"):
            if dev.id == devToProcess:
                device = indigo.devices[dev.name]
                if device != None:
                    if pluginAction.props["armType"] == "disarm":
                        self.disarmCamera(device.states["name"])
                    elif pluginAction.props["armType"] == "arm":
                        self.armCamera(device.states["name"])
                    self.sleep(5)
                    return True 


    def getSnap(self, pluginAction, actionId):
        """ Action item to  get a snapshot from a camera"""
        devToProcess = int(pluginAction.props["camera"])
        device = None
        for dev in indigo.devices.iter("self"):
            if dev.id == devToProcess:
                device = indigo.devices[dev.name]
                if device != None:
                    self.getSnapshot(device.states["name"])
                    return True


    def getVids(self, pluginAction, actionId):
        """ Action item to  get recent video clips from all cameras"""
        mins = int(pluginAction.props["sinceMins"])
        self.getVideo(mins)
        return True



    ########################################
    # Menu Item functions
    ######################
    
    def setArming(self, pluginAction, actionId):
        """ Menu item to arm/disarm a specified sync module """
        devToProcess = int(pluginAction["syncmodule"])
        device = None
        for dev in indigo.devices.iter("self"):
            if dev.id == devToProcess:
                device = indigo.devices[dev.name]
                if device != None:
                    if pluginAction["armType"] == "disarm":
                        self.disarmSync(device.states["name"])
                    elif pluginAction["armType"] == "arm":
                        self.armSync(device.states["name"])
                    return True


    def setArmingCam(self, pluginAction, actionId):
        """ Menu item to arm/disarm a specified camera """
        devToProcess = int(pluginAction["camera"])
        device = None
        for dev in indigo.devices.iter("self"):
            if dev.id == devToProcess:
                device = indigo.devices[dev.name]
                if device != None:
                    if pluginAction["armType"] == "disarm":
                        self.disarmCamera(device.states["name"])
                    elif pluginAction["armType"] == "arm":
                        self.armCamera(device.states["name"])
                    return True

